// AnimationenView.cpp : implementation of the CAnimationenView class
//

#include "stdafx.h"
#include "Animationen.h"

#include "AnimationenDoc.h"
#include "AnimationenView.h"
#include "MainFrm.h"

#include "GLUniverse.h"  // Das OpenGL-Animations-Objekt.
#include "GrueneWiese.h" // Das Visual-C++-Animations-Objekt.

#include <gl\gl.h>       // OpenGL-Untersttzung.
#include <gl\glu.h>
#include <mmsystem.h>    // Fr Sounduntersttzung.

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAnimationenView

IMPLEMENT_DYNCREATE(CAnimationenView, CView)

BEGIN_MESSAGE_MAP(CAnimationenView, CView)
	//{{AFX_MSG_MAP(CAnimationenView)
	ON_WM_CREATE()
	ON_WM_TIMER()
	ON_UPDATE_COMMAND_UI(ID_OPENGL_DEMO, OnUpdateOpenglDemo)
	ON_COMMAND(ID_OPENGL_DEMO, OnOpenglDemo)
	ON_UPDATE_COMMAND_UI(ID_CPLUS_DEMO, OnUpdateCplusDemo)
	ON_COMMAND(ID_CPLUS_DEMO, OnCplusDemo)
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
	ON_UPDATE_COMMAND_UI_RANGE(IDB_OPENGL_PLAY, IDB_OPENGL_STOP, OnUpdateGLPalette1)
	ON_COMMAND_RANGE(IDB_OPENGL_PLAY, IDB_OPENGL_STOP, OnGLPalette1)
	ON_UPDATE_COMMAND_UI_RANGE(IDB_OPENGL_PLAY2, IDB_OPENGL_STOP2, OnUpdateGLPalette2)
	ON_COMMAND_RANGE(IDB_OPENGL_PLAY2, IDB_OPENGL_STOP2, OnGLPalette2)
	ON_UPDATE_COMMAND_UI_RANGE(IDB_VC_PLAY, IDB_VC_STOP, OnUpdateVCPalette)
	ON_COMMAND_RANGE(IDB_VC_PLAY, IDB_VC_STOP, OnVCPalette)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CAnimationenView construction/destruction

CAnimationenView::CAnimationenView()
{
	// Ein Objekt von der OpenGL-Animation anlegen.
	m_pGLAni = new COpenGLUniverse();
	// Ein Objekt von der Visual-C++-Animation anlegen.
	m_pVCAni = new CGrueneWiese();
	m_bIsOpenGL    = FALSE; // Keine OpenGL-Demo anzeigen.
	m_bIsCPlusDemo = FALSE; // Keine C++-Demo anzeigen.
}

CAnimationenView::~CAnimationenView()
{
	// Altes OpenGL-Animations-Klassen-Objekt lschen.
	delete m_pGLAni;
	// Altes Visual-C++-Animations-Klassen-Objekt lschen.
	delete m_pVCAni;
	// Sound anhalten (sollte er schon angehalten sein, passiert nichts).
	sndPlaySound(NULL, SND_ASYNC);
}

BOOL CAnimationenView::PreCreateWindow(CREATESTRUCT& cs)
{
	// An OpenGL window must be created with the following flags and must not
    // include CS_PARENTDC for the class style. Refer to SetPixelFormat
    // documentation in the "Comments" section for further information.
    cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CAnimationenView drawing

void CAnimationenView::OnDraw(CDC* pDC)
{
	ASSERT_VALID(pDoc);

	// Soll die OpenGL-Demo angezeigt werden oder sollen Objekte gezeichnet werden knnen.
	if(m_bIsOpenGL)
	{
		// Das jeweilige (aktuelle) Bild der Animation anzeigen.
		m_pGLAni->Animation();
		return;
	}
	// Soll die C++-Animation angezeigt werden oder sollen Objekte gezeichnet werden knnen.
	if(m_bIsCPlusDemo)
	{
		m_pVCAni->Animation();
		return;
	}

	// Wenn keine Animation angezeigt wird, soll der Hintergrund des Fensters schwarz sein.
	CRect crSichtgroesse;	
	// Gre der Sicht besorgen.
	GetClientRect(crSichtgroesse);
	// Schwarz.
	CBrush  cbBrush(COLORREF(0x00000000));
	// Neuen Zeichen-Brush auswhlen (alten merken).
	CBrush* pOldBrush = pDC->SelectObject(&cbBrush); 
	pDC->FillRect(crSichtgroesse, &cbBrush);
	// Alten Brush wiederherstellen.
	pDC->SelectObject(pOldBrush);
}

/////////////////////////////////////////////////////////////////////////////
// CAnimationenView diagnostics

#ifdef _DEBUG
void CAnimationenView::AssertValid() const
{
	CView::AssertValid();
}

void CAnimationenView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CAnimationenDoc* CAnimationenView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CAnimationenDoc)));
	return (CAnimationenDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CAnimationenView message handlers

// C++-Animation.
void CAnimationenView::OnCplusDemo() 
{
	// Zeiger auf den MAIN-Frame, um die Paletten aus- bzw. einblenden zu knnen.
	CMainFrame* pFrame  = (CMainFrame*) AfxGetApp()->m_pMainWnd;
	// Flag fr die Demo togglen.
	m_bIsCPlusDemo = ~m_bIsCPlusDemo;
	// Palettenansicht schalten.
	if(m_bIsCPlusDemo)
	{
		// Der standardmig aktivierte Button der Palette der VC-Animation
		// ist der PLAY-Schalter (dazu mu also der STOP-Button deaktiviert werden). 
		m_uiWelcherVCButton = IDB_VC_STOP;
		// Die Palette der C++-Animation wird angezeigt und mit RecalcLayout wird 
		// dies dem MAIN-Frame mitgeteilt.
		pFrame->m_wndVCBox.ShowWindow(SW_SHOW);
		pFrame->RecalcLayout();
		// Gre des Fensters besorgen.
		RECT rectWindow;
		GetClientRect(&rectWindow);
		// Aufbau des ersten FRAMES der Animation.
		m_pVCAni->InitialisiereAnimation(rectWindow);
	}
	else
	{
		// Die Palette der C++-Animation wird verborgen.
		pFrame->m_wndVCBox.ShowWindow(SW_HIDE);
		pFrame->RecalcLayout();
		// Sicht verstndigen.
		Invalidate();
	}
}
// Menbefehlbehandler "C++-Demo".
void CAnimationenView::OnUpdateCplusDemo(CCmdUI* pCmdUI) 
{
	// Befehl ist aktiviert, wenn GL-Demo nicht angezeigt wird, sonst deaktiviert.
	if(m_bIsOpenGL)
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable();
	// Wenn Hkchen dann Demo, sonst keine Demo.
	if(m_bIsCPlusDemo)
		pCmdUI->SetCheck();
	else
		pCmdUI->SetCheck(0);	
}
// Visual-C++-Animation (Palettenteil, Buttonbehandler).
void CAnimationenView::OnUpdateVCPalette(CCmdUI* pCmdUI)
{
	// Es ist immer nur ein Button aktiviert (abhngig vom Zustand der Animation).
	// Den Button aktivieren, der nicht gedrckt wurde.
	pCmdUI->Enable(!(m_uiWelcherVCButton == pCmdUI->m_nID));
}
void CAnimationenView::OnVCPalette(UINT nID)
{
	m_uiWelcherVCButton = nID;
}

// OpenGL-Animation (1. Palettenteil (Planetensystem), Buttonbehandler).
void CAnimationenView::OnUpdateGLPalette1(CCmdUI* pCmdUI)
{
	// Es ist immer nur ein Button aktiviert (abhngig vom Zustand der Animation).
	// Den Button aktivieren, der nicht gedrckt wurde.
	pCmdUI->Enable(!(m_uiWelcherGLButton1 == pCmdUI->m_nID));
}
void CAnimationenView::OnGLPalette1(UINT nID)
{
	m_uiWelcherGLButton1 = nID;
	// Abhngig vom gedrckten Button wird eine entsprechende Aktion ausgelst.
	if(nID == IDB_OPENGL_STOP)
		// Animation anhalten.
		m_bGLAniPlaneten = FALSE;
	// Animation abspielen.
	else if(nID == IDB_OPENGL_PLAY)
		// Starten der Animation (durch FLAG setzen).
		m_bGLAniPlaneten = TRUE;
}
// OpenGL-Animation (2. Palettenteil (Beobachterstandpunkt), Buttonbehandler).
void CAnimationenView::OnUpdateGLPalette2(CCmdUI* pCmdUI)
{
	// Es ist immer nur ein Button aktiviert (abhngig vom Zustand der Animation).
	// Den Button aktivieren, der nicht gedrckt wurde.
	pCmdUI->Enable(!(m_uiWelcherGLButton2 == pCmdUI->m_nID));
}
void CAnimationenView::OnGLPalette2(UINT nID)
{
	m_uiWelcherGLButton2 = nID;
	// Abhngig vom gedrckten Button wird eine entsprechende Aktion ausgelst.
	if(nID == IDB_OPENGL_STOP2)
	{
		// Animation anhalten.
		m_bGLAniBeobachter = FALSE;
		// Sound anhalten.
		sndPlaySound(NULL, SND_ASYNC);
	}
	// Animation abspielen.
	else if(nID == IDB_OPENGL_PLAY2)
	{
		// Starten der Animation (durch FLAG setzen).
		m_bGLAniBeobachter = TRUE;
		// Sound starten.
		sndPlaySound("sterne.wav", SND_NODEFAULT | SND_ASYNC | SND_LOOP | SND_NOSTOP);
	}
}
// Menbefehlbehandler "OpenGL-Demo".
void CAnimationenView::OnUpdateOpenglDemo(CCmdUI* pCmdUI) 
{
	// Befehl aktivieren, wenn C++-Demo nicht angezeigt wird.
	if(m_bIsCPlusDemo)
		pCmdUI->Enable(FALSE);
	else
		pCmdUI->Enable();
	// Wenn Hkchen dann Demo, sonst keine Demo.
	if(m_bIsOpenGL)
		pCmdUI->SetCheck();
	else
		pCmdUI->SetCheck(0);
}
// Menbefehlbehandler "OpenGL-Demo".
void CAnimationenView::OnOpenglDemo() 
{
	// Zeiger auf den MAIN-Frame, um die Paletten aus- bzw. einblenden zu knnen.
	CMainFrame* pFrame  = (CMainFrame*) AfxGetApp()->m_pMainWnd;
	// Flag fr die Demo togglen.
	m_bIsOpenGL = ~m_bIsOpenGL;
	// Palettenansicht schalten.
	if(m_bIsOpenGL)
	{
		// Der standardmig aktivierte Button der Palette1 der GL-Animation
		// ist der PLAY_Schalter (dazu mu also der STOP-Button deaktiviert werden). 
		m_uiWelcherGLButton1 = IDB_OPENGL_STOP;
		// Der standardmig aktivierte Button der Palette2 der GL-Animation
		// ist der PLAY_Schalter (dazu mu also der STOP-Button deaktiviert werden). 
		m_uiWelcherGLButton2 = IDB_OPENGL_STOP2;
		// Die Palette fr die Animation wird angezeigt und mit RecalcLayout
		// wird dies dem MAIN-Frame mitgeteilt..
		pFrame->m_wndOpenGLBox.ShowWindow(SW_SHOW);
		pFrame->RecalcLayout();
		// Die GL-Animation wird vorbereitet.
		if(!m_pGLAni->InitialisiereGL())
		{
			AfxMessageBox("Kann OpenGL-Animation nicht abspielen");
			return;
		}
		// Kleineren Ausschnitt der Sicht fr die GL-Animation bereitstellen.
		glViewport((m_iViewSizeX - 420) / 2, (m_iViewSizeY - 315) / 2, 420, 315);
		glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
		glClear(GL_COLOR_BUFFER_BIT);

		m_pGLAni->InitialisiereAnimation();
		// Startinformationen (Planeten-Koordinaten) fr die Animation bereitstellen.
		m_pGLAni->m_iP1Winkelanzahl = 0;
		m_pGLAni->m_iP2Winkelanzahl = 0;
		m_pGLAni->m_iP3Winkelanzahl = 0;
		m_pGLAni->m_iP4Winkelanzahl = 0;
		m_pGLAni->m_fP1Winkel = 0.1745329252f; // Winkel von 10.
		m_pGLAni->m_fP2Winkel = 0.3490658504f; // Winkel von 20.
		m_pGLAni->m_fP3Winkel = 0.1047197551f; // Winkel von  6.
		m_pGLAni->m_fP4Winkel = 0.05235987756f;// Winkel von  3.
		// Augpunktkoordinaten.
		m_pGLAni->m_fvpx =  0.0f;
		m_pGLAni->m_fvpz = 25.0f;
		m_pGLAni->m_frpx =  0.0f;
		m_pGLAni->m_frpz =  0.0f;
		// Augpunktposition (welchen Standpunkt hat der Augpunkt gerade) initialisieren.
		m_iOpenGLBeobachterposition = 0;
		// Noch keine Animation starten.
		m_bGLAniPlaneten   = FALSE;
		m_bGLAniBeobachter = FALSE;
		// Jedoch der Timer wird schon gestartet.
		m_uiTimer = SetTimer(OPENGLANIMATION, 150, NULL);
	}
	else
	{
		// Die Palette fr die Animation wird versteckt.
		pFrame->m_wndOpenGLBox.ShowWindow(SW_HIDE);
		pFrame->RecalcLayout();
		// Der Timer wird gekillt, damit nicht unntigerweise CPU-Zeit verbraucht wird.
		// Animation anhalten.
		KillTimer(OPENGLANIMATION);
		// Sound anhalten (sollte er schon angehalten sein, passiert nichts).
		sndPlaySound(NULL, SND_ASYNC);
	}
}

// Hier wird der GL-Support initialisiert bzw. die Initialisierung aufgerufen.
int CAnimationenView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// Der OpenGL-Animation wird hier der Gertekontext mitgeteilt.
	// Das gleiche geschieht mit der Visual-C++-Animations-Klasse.
	m_pDC = new CClientDC(this);
	m_pGLAni->Create(m_pDC);
	m_pVCAni->Create(m_pDC);
	
	return 0;
}

// Zustndig fr das abspielen der OpenGL- und C++-Animation.
void CAnimationenView::OnTimer(UINT nIDEvent) 
{
	// Fr die OpenGL-Animation werden je Timeraufruf vernderte Tiefeninformationen errechnet
	// und an die entsprechende Funktion bergeben.
	if(nIDEvent == OPENGLANIMATION)
	{
		// Rotation der Planeten.
		if(m_bGLAniPlaneten)
		{
			// Rotieren des 1. Planeten.
			if(m_pGLAni->m_iP1Winkelanzahl < 35)
				m_pGLAni->m_iP1Winkelanzahl++;
			else
				m_pGLAni->m_iP1Winkelanzahl = 0;
			// Rotieren des 2. Planeten.
			if(m_pGLAni->m_iP2Winkelanzahl < 17)
				m_pGLAni->m_iP2Winkelanzahl++;
			else
				m_pGLAni->m_iP2Winkelanzahl = 0;
			// Rotieren des 3. Planeten.
			if(m_pGLAni->m_iP3Winkelanzahl < 59)
				m_pGLAni->m_iP3Winkelanzahl++;
			else
				m_pGLAni->m_iP3Winkelanzahl = 0;
			// Rotieren des 4. Planeten.
			if(m_pGLAni->m_iP4Winkelanzahl < 119)
				m_pGLAni->m_iP4Winkelanzahl++;
			else
				m_pGLAni->m_iP4Winkelanzahl = 0;
		}

		// Bewegung des Beobachters durch den Raum.
		if(m_bGLAniBeobachter)
		{
			// Um 36 Z-Units wird sich in die Tiefe bewegt, anschlieend nach links um 180
			// gedreht und zurckgeflogen. Dort wieder um 180 nach links drehen und von vorn.
			// Um die jeweilige Lage analysieren zu knnen wird ein Zhlerstand benutzt.
			// Hinflug.
			if(m_iOpenGLBeobachterposition < 180)
			{
				m_pGLAni->m_fvpz -= 0.2f;  // Die Tiefenwerte werden gendert.
				m_pGLAni->m_frpz -= 0.2f;
			}
			// 1. Drehsequenz.
			if(m_iOpenGLBeobachterposition > 179 && m_iOpenGLBeobachterposition < 270)
			{
				m_pGLAni->m_frpz += (1.0f / 3.0f);  // Tiefenwert und X-Koordinate des
				m_pGLAni->m_frpx -= (1.0f / 3.0f);  // Blickpunktes werden gendert.
			}
			if(m_iOpenGLBeobachterposition > 269 && m_iOpenGLBeobachterposition < 360)
			{
				m_pGLAni->m_frpz += (1.0f / 3.0f);  // Tiefenwert und X-Koordinate des
				m_pGLAni->m_frpx += (1.0f / 3.0f);  // Blickpunktes werden gendert.
			}
			// Rckflug.
			if(m_iOpenGLBeobachterposition > 359 && m_iOpenGLBeobachterposition < 540)
			{
				m_pGLAni->m_fvpz += 0.2f;  // Die Tiefenwerte werden gendert.
				m_pGLAni->m_frpz += 0.2f;
			}
			// 2. Drehsequenz.
			if(m_iOpenGLBeobachterposition > 539 && m_iOpenGLBeobachterposition < 630)
			{
				m_pGLAni->m_frpz -= (1.0f / 3.0f);  // Tiefenwert und X-Koordinate des
				m_pGLAni->m_frpx -= (1.0f / 3.0f);  // Blickpunktes werden gendert.
			}
			if(m_iOpenGLBeobachterposition > 629 && m_iOpenGLBeobachterposition < 720)
			{
				m_pGLAni->m_frpz -= (1.0f / 3.0f);  // Tiefenwert und X-Koordinate des
				m_pGLAni->m_frpx += (1.0f / 3.0f);  // Blickpunktes werden gendert.
			}
			// Von vorne beginnen.
			if(m_iOpenGLBeobachterposition == 720)
				m_iOpenGLBeobachterposition = 0;
			else
				m_iOpenGLBeobachterposition++;       // Nchstes Bild.
		}

		// Neues Bild anzeigen (wenn erforderlich).
		if(m_bGLAniPlaneten || m_bGLAniBeobachter)
			m_pGLAni->Animation();
	}
	
	CView::OnTimer(nIDEvent);
}

// Bei jeder nderung der Sicht wird ihre Gre gespeichert.
void CAnimationenView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);	
	// Die Gre der Sicht merken fr den OpenGL-Viewport.
	m_iViewSizeX = cx;
	m_iViewSizeY = cy;	
}
