// Implementierungsdatei einer OpenGL-Animation (Universum).

#include "stdafx.h"
#include "GLUniverse.h"

#include <gl\gl.h>  // OpenGL-Untersttzung.
#include <gl\glu.h>
#include <math.h>   // Fr RANDOM-Zahl.

// Konstruktor.
COpenGLUniverse::COpenGLUniverse()
{
	// Die Listennummern initialisieren.
	m_iSterne       = 1;       
	
	m_iSonne        = 2;
	
	m_iMilchstrasse = 12;

	m_iAnimation1   = 20;       // Listennummern fr die Animationen.
	m_iAnimation2   = 21;

	// Augpunktkoordinaten einstellen (nur die vernderbaren Komponenten).
	m_fvpx =  0.0f;
	m_fvpz = 25.0f;
	m_frpx =  0.0f;
	m_frpz =  0.0f;
}
// Destruktor.
COpenGLUniverse::~COpenGLUniverse()
{

}
// Create-Methode initialisiert den DC-Zeiger.
void COpenGLUniverse::Create(CClientDC* pDC)
{
	m_pDC = pDC;  // DC fr die OpenGL-Initialisierung.
}

// Erzeugen einer Menge von Sternen.
void COpenGLUniverse::Sterne(int iAnzahl, float fRaumausdehnung)
{
	float fRaRa = float(2.0 * fRaumausdehnung);

	// Die Farbe der greren Objekte (Sterne) wird zufllig ermittelt.
	// Konstante RAND_MAX enthlt maximal gre Random-Zahl
	float fMaterial[3] = {float(rand() * 1.0 / RAND_MAX), 
		                  float(rand() * 1.0 / RAND_MAX),
						  float(0.6 * rand() * 1.0 / RAND_MAX)};

	glNewList(m_iSterne, GL_COMPILE);		
		glPushAttrib(GL_ALL_ATTRIB_BITS);
		glPointSize(3.0f);
		glBegin(GL_POINTS);
		for(int i = 0; i < iAnzahl; i++)
		{
			glMaterialfv(GL_FRONT, GL_EMISSION, fMaterial);
			glVertex3f(float(rand() * fRaRa / RAND_MAX - fRaumausdehnung),
				       float(rand() * fRaRa / RAND_MAX - fRaumausdehnung),
					   float(rand() * fRaRa / RAND_MAX - fRaumausdehnung));
		}
		glEnd();
	glEndList();
	glPopAttrib();
}
// Erzeugen der Milchstrae.
void COpenGLUniverse::Milchstrasse(int iAnzahl, float fRaumausdehnung)
{
	float fRaRa = float(2.0 * fRaumausdehnung);
	float fMaterial[4] = {1.0f, 1.0f, 1.0f, 1.0f};

	glNewList(m_iMilchstrasse, GL_COMPILE);
		glPushAttrib(GL_ALL_ATTRIB_BITS);
		glPointSize(1.0f);
		glBegin(GL_POINTS);
		for(int i = 0; i < iAnzahl; i++)
		{
			glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, fMaterial);
			glVertex3f(float(rand() * fRaRa / RAND_MAX - fRaumausdehnung),
				       float(rand() * fRaRa / RAND_MAX - fRaumausdehnung),
					   float(rand() * fRaRa / RAND_MAX - fRaumausdehnung));
		}
		glEnd();
	glEndList();
	glPopAttrib();
}
// Erzeugen der Sonne.
void COpenGLUniverse::Sonne()
{
	GLUquadricObj* qSun = gluNewQuadric();
	float fMaterial1[4] = {1.0f, 1.0f, 0.0f, 1.0f};
	float fMaterial2[4] = {1.0f, 1.0f, 0.0f, 1.0f};

	glNewList(m_iSonne, GL_COMPILE);
		glEnable(GL_CULL_FACE);
		glPushAttrib(GL_ALL_ATTRIB_BITS);
		glMaterialfv(GL_FRONT, GL_EMISSION, fMaterial1);
		glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, fMaterial2);
		gluSphere(qSun, 2.0f, 20, 20);
		glPopAttrib();
	glEndList();
}
// Einstellen der Lichtquelle (Sonne).
void COpenGLUniverse::Sonnenlicht()
{
	float fLicht1[4] = {0.0f, 0.0f, 0.0f, 1.0f};
	float fLicht2[4] = {1.0f, 1.0f, 1.0f, 1.0f};

	glLightfv(GL_LIGHT1, GL_POSITION, fLicht1);
	glLightfv(GL_LIGHT1, GL_DIFFUSE, fLicht2);
}
// Erster Planet.
void COpenGLUniverse::Planet1()
{
	GLUquadricObj* qPlanet = gluNewQuadric();
	float fMaterial1[4] = {0.0f, 0.0f, 0.0f, 1.0f};
	float fMaterial2[4] = {0.1f, 0.8f, 0.1f, 1.0f};  // Grn-Ton.
	float fMaterial3[4] = {0.0f, 0.0f, 0.0f, 1.0f};
	
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glMaterialfv(GL_FRONT, GL_EMISSION, fMaterial1);
	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, fMaterial2);
	glMaterialfv(GL_FRONT, GL_SPECULAR, fMaterial3); 
	// Position des Planeten.
	glTranslatef(2.2f * ((-2.6f) * float(cos(m_iP1Winkelanzahl * m_fP1Winkel)) + (-3.1f) * float(sin(m_iP1Winkelanzahl * m_fP1Winkel))),
		         0.0f,
				 2.2f * (2.6f * float(sin(m_iP1Winkelanzahl * m_fP1Winkel)) + (-3.1f) * float(cos(m_iP1Winkelanzahl * m_fP1Winkel))));
	gluSphere(qPlanet, 0.6f, 20, 20);   // Gre und Feinheit des Planeten.
	glPopAttrib();
	glLoadIdentity();	
}
// Zweiter Planet.
void COpenGLUniverse::Planet2()
{
	GLUquadricObj* qPlanet = gluNewQuadric();
	GLUquadricObj* qd1     = gluNewQuadric();
	GLUquadricObj* qd2     = gluNewQuadric();
	GLUquadricObj* qd3     = gluNewQuadric();
	float fMaterial1[4] = {0.0f, 0.0f, 0.0f, 1.0f};
	float fMaterial2[4] = {1.0f, 0.2f, 0.1f, 1.0f};  // Rot-Ton.
	float fMaterial3[4] = {0.0f, 0.0f, 0.0f, 1.0f};
	float fMaterial4[4] = {1.0f, 0.5f, 0.0f, 1.0f};  // Gelb-Ton.
	float fMaterial5[4] = {1.0f, 0.5f, 0.25f, 1.0f}; // Ocker-Ton.


	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glMaterialfv(GL_FRONT, GL_EMISSION, fMaterial1);
	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, fMaterial2);
	glMaterialfv(GL_FRONT, GL_SPECULAR, fMaterial3);
	// Position des Planeten.
	glTranslatef(1.5f * (3.05f * float(cos(m_iP2Winkelanzahl * m_fP2Winkel)) + (-1.7f) * float(sin(m_iP2Winkelanzahl * m_fP2Winkel))), 
				 0.0f, 
				 1.5f * ((-3.05f) * float(sin(m_iP2Winkelanzahl * m_fP2Winkel)) + (-1.7f) * float(cos(m_iP2Winkelanzahl * m_fP2Winkel))));
	gluSphere(qPlanet, 0.7f, 20, 20); // Gre und Feinheit des Planeten.
	glRotatef(45.0f, 0.5f, 0.5f, 0.0f);  // Stellung des Planeten.
	glDisable(GL_CULL_FACE);  // Die Rckseite der Ringe sollen natrlich angezeigt werden,
	// 1. Ring.
	glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, fMaterial4);
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, fMaterial4);
	glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 100.0f);
	gluDisk(qd1, 0.9f, 1.2f, 30, 1);
	// 2. Ring.
	glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, fMaterial5);
	glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE, fMaterial5);
	glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 100.0f);
	gluDisk(qd2, 1.4f, 1.8f, 30, 1);

	glEnable(GL_CULL_FACE);
	glPopAttrib();
	glLoadIdentity();
}
// Dritter Planet.
void COpenGLUniverse::Planet3()
{
	GLUquadricObj* qPlanet = gluNewQuadric();
	float fMaterial1[4] = {0.0f, 0.0f, 0.0f, 1.0f};
	float fMaterial2[4] = {0.0f, 1.0f, 1.0f, 1.0f};  // Cyan.
	float fMaterial3[4] = {0.0f, 0.0f, 0.0f, 1.0f};
	
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glMaterialfv(GL_FRONT, GL_EMISSION, fMaterial1);
	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, fMaterial2);
	glMaterialfv(GL_FRONT, GL_SPECULAR, fMaterial3); 
	// Position des Planeten.
	glTranslatef(2 * (0.0f * float(cos(m_iP3Winkelanzahl * m_fP3Winkel)) + (-5.5f) * float(sin(m_iP3Winkelanzahl * m_fP3Winkel))),
		         0.0f,
				 2 * ((-0.0f) * float(sin(m_iP3Winkelanzahl * m_fP3Winkel)) + (-5.5f) * float(cos(m_iP3Winkelanzahl * m_fP3Winkel))));
	gluSphere(qPlanet, 1.8f, 20, 20);   // Gre und Feinheit des Planeten.
	glPopAttrib();
	glLoadIdentity();
}
// Vierter Planet.
void COpenGLUniverse::Planet4()
{
	GLUquadricObj* qPlanet = gluNewQuadric();
	float fMaterial1[4] = {0.0f, 0.0f, 0.0f, 1.0f};
	float fMaterial2[4] = {1.0f, 0.5f, 0.0f, 1.0f};  // Rot-Grn-Ton.
	float fMaterial3[4] = {0.0f, 0.0f, 0.0f, 1.0f};
	
	glPushAttrib(GL_ALL_ATTRIB_BITS);
	glMaterialfv(GL_FRONT, GL_EMISSION, fMaterial1);
	glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, fMaterial2);
	glMaterialfv(GL_FRONT, GL_SPECULAR, fMaterial3); 

	// Spezielle Berechnung der Y-Koordinate.
	float fY;
	if(m_iP4Winkelanzahl < 60)
		fY = 6.0f - m_iP4Winkelanzahl * 0.2f;
	else
		fY = -6.0f + (m_iP4Winkelanzahl - 60) * 0.2f;
	// Position des Planeten.
	glTranslatef(1 * (20.0f * float(cos(m_iP4Winkelanzahl * m_fP4Winkel)) + (0.0f) * float(sin(m_iP4Winkelanzahl * m_fP4Winkel))),
		         fY,
				 1 * ((-20.0f) * float(sin(m_iP4Winkelanzahl * m_fP4Winkel)) + (0.0f) * float(cos(m_iP4Winkelanzahl * m_fP4Winkel))));
	gluSphere(qPlanet, 1.6f, 20, 20);   // Gre und Feinheit des Planeten.
	glPopAttrib();
	glLoadIdentity();
}
// Darstellung des aktuellen FRAME der Animation.
void COpenGLUniverse::Animation()
{
	// Die Ausgangsstellung.
	glCallList(m_iAnimation1);
	// Neuer Blickpunkt, Blickrichtung.
	gluLookAt(m_fvpx, 4.5f, m_fvpz, m_frpx, 0.0f, m_frpz, 0.0f, 1.0f, 0.0f);
		
	// Der Weltraum.
	glCallList(m_iAnimation2);

	// Die rotierenden Planeten.
	Planet1();
	Planet2();
	Planet3();
	Planet4();

	// Umschalten der Buffer.
	// Ergebnis am Bildschirm darstellen.
	SwapBuffers(m_pDC->GetSafeHdc());
}
// Erstellen der Animationsdisplaylisten.
void COpenGLUniverse::Animationslisten()
{
	glNewList(m_iAnimation1, GL_COMPILE);
		// Lschen des Bildes.
		glClear(GL_COLOR_BUFFER_BIT);
		// Lschen des Tiefen-Buffers.
		glClear(GL_DEPTH_BUFFER_BIT);
		
		// Transformation einstellen.
		glMatrixMode(GL_PROJECTION);
		glLoadIdentity();
		gluPerspective(100.0f, 1.333f, 0.1f, 60.0f); // Angepasst an den 4. Planeten, durch
		// den Wert 60.0 fr den HINTEN-Wert des Viewing-Volumens.
	glEndList();

	glNewList(m_iAnimation2, GL_COMPILE);
		glMatrixMode(GL_MODELVIEW);
		glLoadIdentity();
		glCallList(m_iSterne);
		glCallList(m_iMilchstrasse);
		glCallList(m_iSonne);
	glEndList();
}
// Alle Objekte der Animation werden initialisiert + OpenGL-Support-Initialisierung.
void COpenGLUniverse::InitialisiereAnimation()
{
	glEnable(GL_LIGHTING);

	// Einstellen der Beleuchtungssituation.
	glEnable(GL_LIGHT1);
	float fLightModel[4] = {0.7f, 0.7f, 0.7f, 1.0f};
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, fLightModel);
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);

	// Der Z-Buffer-Test.
	glEnable(GL_DEPTH_TEST);
	glClear(GL_DEPTH_BUFFER_BIT);

	// Rueckseiten der Kugeln sind nicht ntig.
	glCullFace(GL_BACK);
	glEnable(GL_CULL_FACE);

	// Erzeugen von Sternen, die Performance kann ueber die Anzahl bestimmt werden
	Sterne(100, 65.0f);  // 100 Objekte mit 65 Z-Units Tiefenausdehnung.
	// Erzeugen der Milchstrae.
	Milchstrasse(800, 65.0f);
	// Erzeugen der Sonne.
	Sonne();
	Sonnenlicht();
	// Erzeugen der Planeten.
	Planet1();
	Planet2();
	Planet3();
	Planet4();

	Animationslisten();
}
// Mu vor jedem Abspielen der GL-Animation aufgerufen werden, zur Initialisierung
// des OpenGL-Supports.
BOOL COpenGLUniverse::InitialisiereGL()
{
	if (!SetzePixelFormat())
        return FALSE;

	HGLRC hrc;
	// Kontext fr die OpenGL-Funktionen erstellen. Somit ist es diesen mglich in
	// den bereitgestellten Kontext zu zeichnen und somit das Bild zu Anzeige zu bringen.
	hrc = wglCreateContext(m_pDC->GetSafeHdc());
	// Kontext realisieren.
    wglMakeCurrent(m_pDC->GetSafeHdc(), hrc);

	return TRUE;
}
// Das Pixelformat fr OpenGL-Support mu speziell gesetzt werden.
BOOL COpenGLUniverse::SetzePixelFormat()
{
	static PIXELFORMATDESCRIPTOR pfd = 
	{
        sizeof(PIXELFORMATDESCRIPTOR),  // size of this pfd
        1,                              // version number
        PFD_DRAW_TO_WINDOW |            // support window
        PFD_SUPPORT_OPENGL |            // support OpenGL
        PFD_DOUBLEBUFFER,               // double buffered
        PFD_TYPE_RGBA,                  // RGBA type
        24,                             // 24-bit color depth
        0, 0, 0, 0, 0, 0,               // color bits ignored
        32,                             // 32-Bit Alpha-Buffer (Transparentpuffer).
        0,                              // shift bit ignored
        16,                             // 16-Bit Accumulation-Buffer.
        16, 16, 16, 16,                 // 16-Bit Accum fr RGBA-Werte.
        32,                             // 32-Bit Z-Buffer (Depth-Buffer).
        32,                             // 32-Bit Stencil-Buffer (Schablonenpuffer).
        0,                              // no auxiliary buffer
        PFD_MAIN_PLANE,                 // main layer
        0,                              // Reserviert.
        0, 0, 0                         // layer masks ignored
    };
    int iPixelformat;

    if((iPixelformat = ChoosePixelFormat(m_pDC->GetSafeHdc(), &pfd)) == 0)
    {
        AfxMessageBox("Das Pixelformat fr OpenGL-Support konnte nicht ermittelt werden.");
        return FALSE;
    }
    if (SetPixelFormat(m_pDC->GetSafeHdc(), iPixelformat, &pfd) == FALSE)
    {
        AfxMessageBox("Das Pixelformat fr OpenGL-Support konnte nicht gesetzt werden.");
        return FALSE;
    }

	return TRUE;
}
